/*
 *  main.rs
 * 
 *  Copyright (C) 2020 Qarva
 *  Sun Dro <skalatozishvili@qarva.com>
 * 
 * Client implementation for aQua Web Multipipe 
 * with content download and TS dump functionality
 */

mod qws;
use qws::client::QClient;
use qws::segment::QSegmenter;

use std::time::Duration;
use std::sync::{Arc, Mutex};
use std::io::{Write, Result};
use std::fs::OpenOptions;
use std::io::Error;
use std::thread;

fn print_usage(name: &String) -> Result<()> {
    println!("\nUsage: {} <addr> <connects> <content> <time> <output>", name);
    println!("Example: {} ws://127.0.0.1:8189 4 10826 0 data.ts", name);
    return Err(Error::from_raw_os_error(22));
}

fn dump_payload(path: &str, data: Vec<u8>) -> Result<()>  
{
    let mut file = OpenOptions::new()
        .write(true)
        .append(true)
        .create(true)
        .open(path)
        .expect("Unable to open file");

    file.write_all(&data).expect("Unable to write data");
    Ok(())
}

fn main() -> Result<()>
{
    let args: Vec<String> = std::env::args().collect();
    if args.len() < 6 { return print_usage(&args[0]); }

    let connects: usize = args[2].parse().unwrap();
    let content: u32 = args[3].parse().unwrap();

    let segmenter = Arc::new(Mutex::new(QSegmenter::new(connects)));
    let mut threads = Vec::new();

    for connect in 0..connects {
        let (addr, time, output, segmenter) = (
            args[1].clone(), 
            args[4].clone(), 
            args[5].clone(), 
            segmenter.clone()
        );

        let mut client: QClient = QClient::connect(&addr);
        client.set_connect(connect+1, connects);
        client.set_content(content, &time);

        let handler = thread::spawn(move || {
            client.send_command("play").unwrap();

            loop {
                let mut packet = client.recv_packet();
                let packet_type = packet.get_type();

                if packet_type == "nogop" {
                    std::thread::sleep(Duration::new(1, 0));
                    client.send_command("getgop").unwrap();
                    continue;
                } else if packet_type == "data" {
                    client.send_command("ack").unwrap();
                    if segmenter.lock().unwrap().push_packet(&mut packet) {
                        let mut seg = segmenter.lock().unwrap().pop_segment().unwrap();
                        dump_payload(&output, seg.to_ts_bytes()).unwrap();
                    }
                } else if packet_type == "eos" {
                    if segmenter.lock().unwrap().push_packet(&mut packet) {
                        let mut seg = segmenter.lock().unwrap().pop_segment().unwrap();
                        dump_payload(&output, seg.to_ts_bytes()).unwrap();
                        break;
                    }
                } else if packet_type == "invalid" {
                    println!("Invalid packet: {}", packet.buffer_size);
                    break;
                }
            }
        });

        threads.push(handler);
    }

    while threads.len() > 0 { 
        threads.pop()
        .unwrap()
        .join()
        .unwrap(); 
    }

    Ok(())
}